=begin

= apache/method-dispatcher.rb

Copyright (C) 2004  Shugo Maeda <shugo@modruby.net>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

== Overview

Apache::MethodDispatcher calls Ruby methods.

== Example of httpd.conf

  RubyRequire apache/ruby-run
  <Location /ruby>
  SetHandler ruby-object
  RubyHandler Apache::MethodDispatcher.instance
  </Location>

=end

require "apache/ruby-run"

module Apache
  class MethodDispatcher < RubyRun
    include Singleton

    def handler(r)
      status = check_request(r)
      return status if status != OK
      filename = r.filename.dup.untaint
      require filename
      cls = get_class(filename)
      return NOT_FOUND if cls.nil?
      method_name = get_method_name(r)
      if !cls.public_instance_methods(false).include?(method_name)
        return FORBIDDEN 
      end
      obj = cls.new
      method = obj.method(method_name)
      s = method.arity == 1 ? method.call(r) : method.call
      send_header(r)
      if r.output_buffer.empty? && r.bytes_sent == 0
        r.write(s)
      end
      return OK
    end

    private

    def get_class(filename)
      s = File.basename(filename).sub(/\.[^.]*$/, "")
      class_name = s.gsub(/\G([a-z](?:[A-Za-z]*))-?/) { $1.capitalize }
      begin
        return Object.const_get(class_name)
      rescue NameError
        return nil
      end
    end

    def get_method_name(r)
      if r.path_info.nil? || r.path_info.empty?
        return "index"
      end
      return r.path_info.sub(/^\//, "").untaint
    end

    def send_header(r)
      if !r.sent_http_header?
        if r.content_type.nil?
          if r.default_charset
            r.content_type = "text/html; charset=" + r.default_charset
          else
            r.content_type = "text/html"
          end
        end
        r.send_http_header
      end
    end
  end
end
